--[[
MultiTool Multi Boxing utilities inspired by smurfys MultiBoxer addon

Shouts to http://old.wowace.com/wiki/WelcomeHome_-_Your_first_Ace3_Addon

Extra special shouts to Allara and OrionShock for pointing me in the right direction RE: AceComm-3.0

Author: DigitalSorceress
Date: 2008/11/16
Version: 0.1.06
]]

-- Some initialization of our happy local vars
local myName = "MultiTool"
local myVersion = "0.1.06"

local taxiCommThrottle = false

MultiTool = LibStub("AceAddon-3.0"):NewAddon("MultiTool", "AceConsole-3.0","AceComm-3.0","AceEvent-3.0","AceSerializer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MultiTool")

-- ------------------------------------------------
-- CONFIGURATION STUFF
-- ------------------------------------------------
-- Holds our configuration options
local options = {
  name = "MultiTool",
  handler = MultiTool,
  type = 'group',
  args = {
   debugFlag = {
      type = "toggle",
      name = L["DEBUGGING_NAME"],
      desc = L["DEBUGGING_DESC"],
      get = "getDebugFlag",
      set = "setDebugFlag"
    },
   taxiFlag = {
      type = "toggle",
      name = L["TAXI_NAME"],
      desc = L["TAXI_DESC"],
      get = "getTaxiFlag",
      set = "setTaxiFlag"
    },
   gossipFlag = {
      type = "toggle",
      name = L["GOSSIP_NAME"],
      desc = L["GOSSIP_DESC"],
      get = "getGossipFlag",
      set = "setGossipFlag"
    },   
   rewardFlag = {
      type = "toggle",
      name = L["REWARD_CLONE_NAME"],
      desc = L["REWARD_CLONE_DESC"],
      get = "getRewardFlag",
      set = "setRewardFlag"
    },   
   duelFlag = {
      type = "toggle",
      name = L["DUELING_NAME"],
      desc = L["DUELING_DESC"],
      get = "getDuelFlag",
      set = "setDuelFlag"
    }
  }
}

local defaults = {
  profile = {
    debugFlag = false,
    duelFlag = true,
    taxiFlag = true,
    gossipFlag = true,
    rewardFlag = false
  }
}


--
-- Field handlers
--
function MultiTool:getMyMsg(info)
  return self.db.profile.myMsg
end

function MultiTool:setMyMsg(info, newValue)
  self.db.profile.myMsg = newValue
end

function MultiTool:getShowInChat(info)
    return self.db.profile.showInChat
end

function MultiTool:setShowInChat(info, value)
    self.db.profile.showInChat = value
end

function MultiTool:getDebugFlag(info)
  return self.db.profile.debugFlag
end
  
function MultiTool:setDebugFlag(info, value)
  self.db.profile.debugFlag = value
end

function MultiTool:getShowOnScreen(info)
    return self.db.profile.showOnScreen
end

function MultiTool:setShowOnScreen(info, value)
    self.db.profile.showOnScreen = value
end

function MultiTool:getDuelFlag(info)
  return self.db.profile.duelFlag
end
  
function MultiTool:setDuelFlag(info, value)
  self.db.profile.duelFlag = value
end

function MultiTool:getTaxiFlag(info)
  return self.db.profile.taxiFlag
end
  
function MultiTool:setTaxiFlag(info, value)
  self.db.profile.taxiFlag = value
end

function MultiTool:getGossipFlag(info)
  return self.db.profile.gossipFlag
end
  
function MultiTool:setGossipFlag(info, value)
  self.db.profile.gossipFlag = value
end

function MultiTool:getRewardFlag(info)
  return self.db.profile.rewardFlag
end
  
function MultiTool:setRewardFlag(info, value)
  self.db.profile.rewardFlag = value
end

--
-- Chat Command setup
-- 
function MultiTool:ChatCommand(input)
  if not input or input:trim() == "" then
    -- This would use the Ace config gui
    -- LibStub("AceConfigDialog-3.0"):Open("MultiTool")
    -- This uses the Bliz options pane instead
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(MultiTool, L["SHORT_SLASH_CMD"], L["FULL_SLASH_CMD"], input)
  end
end


-- ------------------------------------------------
-- BASIC REQUIRED FUNCTIONS
-- ------------------------------------------------

-- Called when the addon is loaded
function MultiTool:OnInitialize()
  -- Using the AceDB for storage of our info
  self.db = LibStub("AceDB-3.0"):New("MultiToolDB", defaults, "Default")

  -- main options table stuff
  LibStub("AceConfig-3.0"):RegisterOptionsTable("MultiTool", options)
  -- adding to Blizzard options configuration
  self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MultiTool", "MultiTool")
  -- registering the slash commands
  self:RegisterChatCommand(L["FULL_SLASH_CMD"], "ChatCommand")
  self:RegisterChatCommand(L["SHORT_SLASH_CMD"], "ChatCommand")
  self:RegisterChatCommand("help", "helpDialog")
  
  self:RegisterComm("MultiTool","OnCommReceived")
end


-- Called when the addon is enabled
function MultiTool:OnEnable()
  -- Register Duel requested event so we can swat em
  self:RegisterEvent("DUEL_REQUESTED", "duelRequested")
	
	-- Taxi hooks
	hooksecurefunc("TakeTaxiNode",takeTaxiNodeHook)

  -- Quest gossip hooks
	hooksecurefunc("GetQuestReward", getQuestRewardHook)
	hooksecurefunc("SelectGossipActiveQuest", selectGossipActiveQuestHook)
	hooksecurefunc("SelectAvailableQuest", selectAvailableQuestHook)
	hooksecurefunc("SelectGossipAvailableQuest", selectGossipAvailableQuestHook)
	hooksecurefunc("AcceptQuest", acceptQuestHook)
	hooksecurefunc("SelectGossipOption", selectGossipOptionHook)
	hooksecurefunc("CompleteQuest", completeQuestHook)
	hooksecurefunc("SelectActiveQuest", selectActiveQuestHook)
	hooksecurefunc("DeclineQuest", declineQuestHook)

  if (self.db.profile.debugFlag) then
    self:Print(myName.." v"..myVersion.." ENABLED!")
  end
end


-- Called when the addon is disabled
function MultiTool:OnDisable()
  --
end


-- ------------------------------------------------
-- EVENT HANDLERS
-- ------------------------------------------------

--
-- If profile is set for denying duels, then autocancels all incoming duels
--
function MultiTool:duelRequested(challenger)
	if (self.db.profile.duelFlag) then
    if (self.debugFlag) then
      self:Print("Duel denied from challenger:"..challenger)
    end
    CancelDuel();
  	StaticPopup_Hide("DUEL_REQUESTED")
	end
end


-- ------------------------------------------------
-- HOOKED FUNCTIONS
-- ------------------------------------------------


function selectGossipActiveQuestHook(index, noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "SELECT_GOSSIP_ACTIVE",
      index = index
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end


function getQuestRewardHook(index, noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "GET_QUEST_REWARD",
      index = index
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end


function selectAvailableQuestHook(index, noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "SELECT_AVAILABLE",
      index = index
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end


function selectGossipAvailableQuestHook(index, noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "SELECT_GOSSIP_AVAILABLE",
      index = index
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end


function selectGossipOptionHook(index, noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "SELECT_OPTION",
      index = index
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end


function selectActiveQuestHook(index, noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "SELECT_ACTIVE",
      index = index
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end


function acceptQuestHook(noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "ACCEPT_QUEST"
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end


function completeQuestHook(noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "COMPLETE_QUEST"
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end


function declineQuestHook(noComm)
  local self = MultiTool
  if (self.db.profile.gossipFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "QUEST_GOSSIP",
      sub_op = "DECLINE_QUEST"
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("gossipFlag is false... not sending comms")
    end
  end
end

--
-- Taxi following
--
function takeTaxiNodeHook(index, noComm) 
  local self = MultiTool
    
  -- Only use the additional functionality IF taxiFlag is true
  if (self.db.profile.taxiFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "TAKETAXI",
      node = index,
      nodeName = TaxiNodeName(index)
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"   
    
    -- rpcCommandSender adds checking of the noComm value before sending
    self:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
  else
    if (self.db.profile.debugFlag) then
      self:Print("taxiFlag is false... not sending comms")
    end
  end
end -- end function takeTaxiNodeHook


-- ------------------------------------------------
-- COMM-BASED FUNCTIONS
-- ------------------------------------------------

--
-- Handles received commands
--
function MultiTool:OnCommReceived(prefix, message, distribution, sender)
  local success, o = self:Deserialize(message)
  if (self.db.profile.debugFlag) then
  	self:Print("comm received")
  	self:Print("  prefix: "..prefix)
  	self:Print("  message: "..message)
  	self:Print("  distribution: "..distribution)
  	self:Print("  sender: "..sender)
  	self:Print("  player: "..UnitName("player"))
  	self:Print("  Deserialize success: "..tostring(success))
  	self:Print("  Deserialize o: "..tostring(o))
  	--self:Print("  priority: "..priority)
  end -- end if debugFlag

  -- don't do anyting if we could not deserialize
  if (success == false) then
    if (self.db.profile.debugFlag) then
      self:Print("Failed to deserialize the message - OnCommReceived can not process any further")
    end -- if debugFlag
    return -- Failure
  else
    --
    -- TEMPLATE - Not actually used
    --
    if (o["op"] == "foo") then
      if (self.db.profile.actionFlag) then
        -- Put any extra conditions here
        if ("someCondition" == "some value") then
          if (self.db.profile.debugFlag) then
            -- debug statements for it here
          end -- ending debug

          return -- do nothing
        else
          -- do the action
          if (self.db.profile.debugFlag) then
            self:Print("OPNAME op engaged")
          end -- ending debug
          -- actual command such as TakeTaxiNode(o["node"], "noComm")
        end -- ending extra conditions
      end -- if gossipFlag
    end -- end if op == foo
    
    --
    -- Quest Gossip handler for cloning actions at quest givers
    --
    if (o["op"] == "QUEST_GOSSIP") then
      if (self.db.profile.gossipFlag) then
        -- Put global quest gossip checks here
        if ("someCondition" == "some bad value") then
          -- conditions mean we DON'T do anything
          if (self.db.profile.debugFlag) then
            -- debug statements for it here
            self:Print("Global QUEST_GOSSIP global condition failed - not doing anything")
          end -- ending debug

          return -- do nothing
        else
          -- do the action
          if (self.db.profile.debugFlag) then
            self:Print("QUEST_GOSSIP global conditions succeded - parsing subcommands")
          end -- ending debug
          
          if (o["sub_op"] == "SELECT_GOSSIP_AVAILABLE") then
            -- tacking on noComm so we don't cause message storm
         		SelectGossipAvailableQuest(o["index"], "noComm")
          elseif (o["sub_op"] == "SELECT_AVAILABLE") then
            -- tacking on noComm so we don't cause message storm
         		SelectAvailableQuest(o["index"], "noComm")
          elseif (o["sub_op"] == "SELECT_OPTION") then
            -- tacking on noComm so we don't cause message storm
		        SelectGossipOption(o["index"], "noComm");
          elseif (o["sub_op"] == "SELECT_ACTIVE") then
            -- tacking on noComm so we don't cause message storm
		        SelectActiveQuest(o["index"], "noComm");
          elseif (o["sub_op"] == "ACCEPT_QUEST") then
            -- tacking on noComm so we don't cause message storm
		        AcceptQuest("noComm")
          elseif (o["sub_op"] == "GET_QUEST_REWARD") then
            -- This one's special.. using logic from QuestGuru to make sure we
            -- don't allow cloning of reward choices
            if (self.db.profile.rewardFlag or GetNumQuestChoices() < 2) then
              if (self.db.profile.debugFlag) then
                self:Print(tostring(GetNumQuestChoices()).." choice - ok to run")
              end 
              -- tacking on noComm so we don't cause message storm
    		      GetQuestReward(o["index"],"noComm")	  
            else
              if (self.db.profile.debugFlag) then
                self:Print(tostring(GetNumQuestChoices()).." choices - Blocking getQuestReward")
              end
            end
          elseif (o["sub_op"] == "SELECT_GOSSIP_ACTIVE") then
            -- tacking on noComm so we don't cause message storm
		        SelectGossipActiveQuest(o["index"], "noComm")
          elseif (o["sub_op"] == "COMPLETE_QUEST") then
            -- tacking on noComm so we don't cause message storm
		        CompleteQuest("noComm")
          elseif (o["sub_op"] == "DECLINE_QUEST") then
            -- tacking on noComm so we don't cause message storm
		        DeclineQuest("noComm")
          end


        end -- ending extra conditions
      end -- if gossipFlag
    end -- end if op == foo
    
    --
    -- TAKETAXI
    --
    if (o["op"] == "TAKETAXI") then

      -- Only continue IF the taxiFlag is true
      if (self.db.profile.taxiFlag) then
  
      -- Ok, This is a BIG  bunch of stuff needed to stop a message storm
      -- the idea here is that we do NOT want to continue to try to take taxi
      -- (and thus re-calling the message send)
      -- IF we are the message originator or if the node we picked is invalid
      -- or if we're not currently at a taxi node or if we're currently ON a 
      -- taxi. 

        -- removed   taxiCommthrottle == true condition 
        if (  UnitOnTaxi("player") 
              or o["nodeName"] == "INVALID" 
              or o["nodeName"] ~= TaxiNodeName(o["node"])
              or sender == UnitName("player")
        ) 
        then
          -- safe to turn off the taxi comm throttle cuz we arent sending
          if (self.db.profile.debugFlag) then
            self:Print("TAKETAXI op NOT engaged")
            self:Print("  Am I on taxi? "..tostring(UnitOnTaxi("player")))
            self:Print("  actual node requested: "..tostring(o["node"]).." ("..o["nodeName"]..")")
            self:Print("  effective node requested: "..tostring(o["node"]).." ("..TaxiNodeName(o["node"])..")")
          end
          -- I think I can remove this 
          -- taxiCommThrottle = false
          return -- DON'T DO ANYTHING: this stops a message storm
        else
          if (self.db.profile.debugFlag) then
            self:Print("TAKETAXI op engaged")
            self:Print("  setting taxiCommThrottle to true")
          end
          -- I think I can 
          -- taxiCommThrottle = true
          -- by passing the extra arg "noComm" to the takeTaxiNode, I think I solve the msg storms
          TakeTaxiNode(o["node"], "noComm")
        end -- if taxiCommTrhottle.. else
      else
        self:Print("taxiFlag is false... ignoring TAKETAXI request")
      end -- end if taxiFlag
    end -- end if op==TAKETAXI
  end -- if success==false
end -- function 

--
-- Wraps up Remote procedure call commands with a "nocomm" throttler
--
function MultiTool:rpcCommandSender(prefix, text, distribution, sender, prio, noComm)
    --debug
    if (self.db.profile.debugFlag) then
      self:Print("rpcCommandSender:: engaged")
      self:Print("  prefix: "..tostring(prefix))
      self:Print("  text: "..tostring(text))
      self:Print("  distribution: "..tostring(distribution))
      self:Print("  sender: "..tostring(sender))
      self:Print("  prio: "..tostring(prio))
      self:Print("  noComm flag: "..tostring(noComm))
    end

    -- New throttling - DO NOT send msg unless noComm is nil
    if (noComm) then
      if (self.db.profile.debugFlag) then
        self:Print("noComm flag IS set, NOT sending command")
      end
      -- Teh Gogglez - They do NOTHING!
      return
    else
      if (self.db.profile.debugFlag) then
        self:Print("noComm flag NOT set, sending command")
      end
      self:SendCommMessage(prefix, text, distribution, sender, prio)
    end
  
end

-- ------------------------------------------------
-- UTILITY FUNCTIONS
-- ------------------------------------------------
function MultiTool:helpCommand()
  self:Print("MultTool Help")
  self:Print("")
  self:Print("COMMANDS:")
  self:Print("  /MultiTool help - Show this help menu")
  self:Print("")
end
